
/* Copyright © 2012 Matthew Champion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of mattunderscore.com nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MATTHEW CHAMPION BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

package com.mattunderscore.http.headers;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

/**
 * A parsed accept header field value with a qualification value.
 * 
 * @author Matt Champion
 */
public class QContentType implements HeaderFieldElement, Qualified
{

    private static final int HASH_SEED = 13;
    private static final int HASH_MULT = 5;
    public static final QContentType TEXT_HTML = new QContentType("text", "html",
            null, 1.0);
    public static final QContentType APP_XHTML = new QContentType("application",
            "xhtml+xml", null, 1.0);
    public static final QContentType APP_RDF = new QContentType("application",
            "rdf+xml", null, 1.0);

    private String type;
    private String subType;
    private Map<String, String> parameters;
    private double qualifier;

    public QContentType(String type, String subType,
            Map<String, String> parameters, double qualifier)
    {
        this.type = type;
        this.subType = subType;
        this.parameters = parameters;
        this.qualifier = qualifier;
    }

    /**
     * Parser for HttpServletRequest to QContentType
     * 
     * @param request
     *            The HTTP request to parse
     * @return The List of content types in the accept header.
     * @throws UnParsableHeaderException
     */
    public static List<QContentType> getRequestContentTypes(
            HttpServletRequest request) throws UnParsableHeaderException
    {
        @SuppressWarnings("unchecked")
        Enumeration<String> headerNames = request.getHeaderNames();
        ContentTypeParser parser = new ContentTypeParser();
        List<QContentType> types = new ArrayList<QContentType>();
        while (headerNames.hasMoreElements())
        {
            String headerName = headerNames.nextElement();
            if (parser.isCorrectHeader(headerName))
            {
                @SuppressWarnings("unchecked")
                Enumeration<String> headers = request.getHeaders(headerName);
                while (headers.hasMoreElements())
                {
                    types.addAll(parser.parseAll(headers.nextElement()));
                }
            }
        }
        return types;
    }

    @Override
    public String toString()
    {
        String string = type + "/" + subType + ";q=" + getQualifier();
        if (parameters != null)
        {
            for (String name : parameters.keySet())
            {
                string += ";" + name + "=" + parameters.get(name);
            }
        }
        return string;
    }

    public double getQualifier()
    {
        return qualifier;
    }

    /**
     * Method that returns true if the two content types can be matched against
     * one another. Takes into account wildcard matching.
     * 
     * @param qct
     * @return True if they match
     */
    public boolean sameType(QContentType qct)
    {
        if (type.equals(qct.type) && subType.equals(qct.subType))
        {
            return true;
        }
        else if (type.equals(qct.type)
                && ("*".equals(subType) || "*".equals(qct.subType)))
        {
            return true;
        }
        else if (("*".equals(type) || "*".equals(qct.type))
                && subType.equals(qct.subType))
        {
            return true;
        }
        else 
        {
            return (("*".equals(type) || "*".equals(qct.type))
                    && ("*".equals(subType) || "*".equals(qct.subType)));
        }
    }

    public boolean isAnySubType()
    {
        return "*".equals(subType);
    }

    public boolean isAnyType()
    {
        return "*".equals(type);
    }
    
    @Override
    public int hashCode()
    {
        int hash = HASH_SEED;
        hash = (hash * HASH_MULT) + type.hashCode();
        hash = (hash * HASH_MULT) + subType.hashCode();
        if (parameters != null)
        {
            for (String key : parameters.keySet())
            {
                hash = (hash * HASH_MULT) + key.hashCode();
                hash = (hash * HASH_MULT) + parameters.get(key).hashCode();
            }
        }
        hash = (hash * HASH_MULT) + (int) (qualifier * 10000.0);
        return hash;
    }

    @Override
    public boolean equals(Object o)
    {
        if (o instanceof QContentType)
        {
            QContentType co = (QContentType) o;
            return (this.type.equals(co.type)
                    && this.subType.equals(co.subType) && this.qualifier == co.qualifier);
        }
        else
        {
            return false;
        }
    }
}
