/* Copyright © 2012 Matthew Champion
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 * Neither the name of mattunderscore.com nor the
 names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL MATTHEW CHAMPION BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

package com.mattunderscore.http.headers;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

/**
 * A parsed accept-language header field value with a qualification value.
 * 
 * @author Matt Champion
 */
public class QLanguage implements HeaderFieldElement, Qualified
{
    private static final int HASH_SEED = 3;
    private static final int HASH_MULT = 7;

    private String language;
    private String subLanguage;
    private double qualifier;

    public QLanguage(String language, String subLanguage, double qualifier)
    {
        this.language = language;
        this.subLanguage = subLanguage;
        this.qualifier = qualifier;
    }

    /**
     * Parser for HttpServletRequest to QLanguage.
     * 
     * @param request
     *            The HTTP request to parse
     * @return The list of languages in the accept-language header
     * @throws UnParsableHeaderException
     */
    public static List<QLanguage> getRequestLanguages(HttpServletRequest request)
            throws UnParsableHeaderException
    {
        @SuppressWarnings("unchecked")
        Enumeration<String> headerNames = request.getHeaderNames();
        LanguageParser parser = new LanguageParser();
        List<QLanguage> types = new ArrayList<QLanguage>();
        while (headerNames.hasMoreElements())
        {
            String headerName = headerNames.nextElement();
            if (parser.isCorrectHeader(headerName))
            {
                @SuppressWarnings("unchecked")
                Enumeration<String> headers = request.getHeaders(headerName);
                while (headers.hasMoreElements())
                {
                    types.addAll(parser.parseAll(headers.nextElement()));
                }
            }
        }
        return types;
    }

    @Override
    public String toString()
    {
        if ("*".equals(subLanguage))
        {
            return language + ";q=" + qualifier;
        }
        else
        {
            return language + "-" + subLanguage + ";q=" + qualifier;
        }
    }

    public double getQualifier()
    {
        return qualifier;
    }

    /**
     * Test to see if two accept-language header values match. Supports wildcard
     * matches.
     * 
     * @param lang
     * @return True if match
     */
    public boolean sameLangauge(QLanguage lang)
    {
        if (this.language.equals(lang.language)
                && this.subLanguage.equals(lang.subLanguage))
        {
            return true;
        }
        else if (this.language.equals(lang.language)
                && ("*".equals(this.subLanguage) || "*"
                        .equals(lang.subLanguage)))
        {
            return true;
        }
        else
        {
            return "*".equals(this.language) || "*".equals(lang.language);
        }
    }

    @Override
    public int hashCode()
    {
        int hash = HASH_SEED;
        hash = (hash * HASH_MULT) + language.hashCode();
        hash = (hash * HASH_MULT) + subLanguage.hashCode();
        hash = (hash * HASH_MULT) + (int) (qualifier * 10000.0);
        return hash;
    }

    @Override
    public boolean equals(Object o)
    {
        if (o instanceof QLanguage)
        {
            QLanguage co = (QLanguage) o;
            return (this.language.equals(co.language)
                    && this.subLanguage.equals(co.subLanguage) && this.qualifier == co.qualifier);
        }
        else
        {
            return false;
        }
    }
}
